﻿using System;
using Ozeki.Media;
using Ozeki.VoIP;

namespace FrequencyToneDetection
{
    class Program
    {
        static ISoftPhone softphone;   // softphone object  
        static IPhoneLine phoneLine;   // phoneline object
        static IPhoneCall call;         // call object

        static PhoneCallAudioReceiver phoneCallAudioReceiver;
        static MediaConnector mediaConnector;
        static FrequencyToneDetector frequencyToneDetector;

        private static void Main(string[] args)
        {
            // Create a softphone object with RTP port range 5000-10000
            softphone = SoftPhoneFactory.CreateSoftPhone(5000, 10000);

            foreach (var s in softphone.Codecs)
            {
                softphone.DisableCodec(s.PayloadType);
            }
            softphone.EnableCodec(8); //only PCMA codec enabled

            // SIP account registration data, (supplied by your VoIP service provider)
            var registrationRequired = true;
            var userName = "sipusername"; //1001
            var displayName = "sipdisplayname"; //1001
            var authenticationId = "authenticationid"; //1001
            var registerPassword = "Password"; //1001
            var domainHost = "pbxip.voipprovider.com";
            var domainPort = 5060;

            var account = new SIPAccount(registrationRequired, displayName, userName, authenticationId, registerPassword, domainHost, domainPort);

            // Send SIP regitration request
            RegisterAccount(account);

            mediaConnector = new MediaConnector();
            phoneCallAudioReceiver = new PhoneCallAudioReceiver();

            frequencyToneDetector = new FrequencyToneDetector();
            frequencyToneDetector.FrequencyDetected += frequencyToneDetector_FrequencyDetected;
            frequencyToneDetector.FrequencyNotFound += frequencyToneDetector_FrequencyNotFound;

            // prevents the termination of the application  
            Console.ReadLine();
        }

        static void frequencyToneDetector_FrequencyNotFound(object sender, EventArgs e)
        {
            Console.WriteLine("Frequency not found!");
        }

        static void frequencyToneDetector_FrequencyDetected(object sender, EventArgs e)
        {
            Console.WriteLine("Frequency detected!");
        }

        static void RegisterAccount(SIPAccount account)
        {
            try
            {
                phoneLine = softphone.CreatePhoneLine(account);
                softphone.IncomingCall += softphone_IncomingCall; // subscribing to the event to get notified about incoming calls
                phoneLine.RegistrationStateChanged += sipAccount_RegStateChanged;
                softphone.RegisterPhoneLine(phoneLine);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error during SIP registration: " + ex);
            }
        }

        private static void sipAccount_RegStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            if (e.State == RegState.Error)
                Console.WriteLine("Registration failed!");

            if (e.State == RegState.RegistrationSucceeded)
                Console.WriteLine("Registration succeeded - Online!");
        }

        private static void softphone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            call = e.Item;  // initializes the call
            call.CallStateChanged += call_CallStateChanged; // subscribes to the event to get notified about the call's states
            
            call.Answer();  // accepts the call (sends back the 200 OK SIP message)
        }

        static void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            Console.WriteLine("Call state: {0}.", e);

            if (e.State == CallState.Answered)
                SetupDevices(); // set up the devices by calling the proper method

            if (e.State.IsCallEnded())
                StopDevices(); // detach and disconnect the devices by calling the proper method
        }

        static void SetupDevices()
        {
            phoneCallAudioReceiver.AttachToCall(call); // attach the phoneCallAudioSender to the call
            mediaConnector.Connect(phoneCallAudioReceiver, frequencyToneDetector);
            frequencyToneDetector.Detect(1400, 6000); // 1400Hz in 6 second
        }

        static void StopDevices()
        {
            mediaConnector.Disconnect(phoneCallAudioReceiver, frequencyToneDetector);
            phoneCallAudioReceiver.Detach();
        }
    }
}